@@ -1,5 +1,5 @@
 /* estream.c - Extended Stream I/O Library
- * Copyright (C) 2004, 2005, 2006, 2007 g10 Code GmbH
+ * Copyright (C) 2004, 2005, 2006, 2007, 2009 g10 Code GmbH
  *
  * This file is part of Libestream.
  *
@@ -91,12 +91,6 @@ typedef void (*func_free_t) (void *mem);
 
 
 
-/* Macros.  */
-
-#define BUFFER_ROUND_TO_BLOCK(size, block_size) \
-
-
-
 /* Locking.  */
 
 #ifdef HAVE_PTH
@@ -138,8 +132,8 @@ dummy_mutex_call_int (estream_mutex_t mutex)
 /* Primitive system I/O.  */
 
 #ifdef HAVE_PTH
-# define ESTREAM_SYS_READ  pth_read
-# define ESTREAM_SYS_WRITE pth_write
+# define ESTREAM_SYS_READ  es_pth_read
+# define ESTREAM_SYS_WRITE es_pth_write
 #else
 # define ESTREAM_SYS_READ  read
 # define ESTREAM_SYS_WRITE write
@@ -231,7 +225,7 @@ static estream_mutex_t estream_list_lock;
   while (0)
 
 
-/* Malloc wrappers to overcvome problems on some older OSes.  */
+/* Malloc wrappers to overcome problems on some older OSes.  */
 static void *
 mem_alloc (size_t n)
 {
@@ -325,6 +319,45 @@ es_list_iterate (estream_iterator_t iterator)
   return ret;
 }
 
+
+
+/*
+ * I/O Helper
+ *
+ * Unfortunately our Pth emulation for Windows expects system handles
+ * for pth_read and pth_write.  We use a simple approach to fix this:
+ * If the function returns an error we fall back to a vanilla read or
+ * write, assuming that we do I/O on a plain file where the operation
+ * can't block.
+ */
+#ifdef HAVE_PTH
+static int
+es_pth_read (int fd, void *buffer, size_t size)
+{
+# ifdef HAVE_W32_SYSTEM
+  int rc = pth_read (fd, buffer, size);
+  if (rc == -1 && errno == EINVAL)
+    rc = read (fd, buffer, size);
+  return rc;
+# else /*!HAVE_W32_SYSTEM*/
+  return pth_read (fd, buffer, size);
+# endif /* !HAVE_W32_SYSTEM*/
+}
+
+static int
+es_pth_write (int fd, const void *buffer, size_t size)
+{
+# ifdef HAVE_W32_SYSTEM
+  int rc = pth_write (fd, buffer, size);
+  if (rc == -1 && errno == EINVAL)
+    rc = write (fd, buffer, size);
+  return rc;
+# else /*!HAVE_W32_SYSTEM*/
+  return pth_write (fd, buffer, size);
+# endif /* !HAVE_W32_SYSTEM*/
+}
+#endif /*HAVE_PTH*/
+
 
 
 /*
@@ -361,11 +394,11 @@ typedef struct estream_cookie_mem
 {
   unsigned int modeflags;	/* Open flags.  */
   unsigned char *memory;	/* Allocated data buffer.  */
-  size_t memory_size;		/* Allocated size of memory.  */
-  size_t memory_limit;          /* Maximum allowed allocation size or
-                                   0 for no limit.  */
+  size_t memory_size;		/* Allocated size of MEMORY.  */
+  size_t memory_limit;          /* Caller supplied maximum allowed
+                                   allocation size or 0 for no limit.  */
   size_t offset;		/* Current offset in MEMORY.  */
-  size_t data_len;		/* Length of data in MEMORY.  */
+  size_t data_len;		/* Used length of data in MEMORY.  */
   size_t block_size;		/* Block size.  */
   struct {
     unsigned int grow: 1;	/* MEMORY is allowed to grow.  */
@@ -375,7 +408,11 @@ typedef struct estream_cookie_mem
 } *estream_cookie_mem_t;
 
 
-/* Create function for memory objects.  */
+/* Create function for memory objects.  DATA is either NULL or a user
+   supplied buffer with the initial conetnt of the memory buffer.  If
+   DATA is NULL, DATA_N and DATA_LEN need to be 0 as well.  If DATA is
+   not NULL, DATA_N gives the allocated size of DATA and DATA_LEN the
+   used length in DATA.  */
 static int
 es_func_mem_create (void *ES__RESTRICT *ES__RESTRICT cookie,
 		    unsigned char *ES__RESTRICT data, size_t data_n,
@@ -388,6 +425,12 @@ es_func_mem_create (void *ES__RESTRICT *ES__RESTRICT cookie,
   estream_cookie_mem_t mem_cookie;
   int err;
 
+  if (!data && (data_n || data_len))
+    {
+      errno = EINVAL;
+      return -1;
+    }
+
   mem_cookie = mem_alloc (sizeof (*mem_cookie));
   if (!mem_cookie)
     err = -1;
@@ -438,6 +481,7 @@ es_func_mem_write (void *cookie, const void *buffer, size_t size)
 {
   estream_cookie_mem_t mem_cookie = cookie;
   ssize_t ret;
+  size_t nleft;
 
   if (!size)
     return 0;  /* A flush is a NOP for memory objects.  */
@@ -447,38 +491,45 @@ es_func_mem_write (void *cookie, const void *buffer, size_t size)
       /* Append to data.  */
       mem_cookie->offset = mem_cookie->data_len;
     }
-	  
-  if (!mem_cookie->flags.grow)
-    {
-      /* We are not alloew to grow, thus limit the size to the left
-         space.  FIXME: Does the grow flag an its semtics make sense
-         at all? */
-      if (size > mem_cookie->memory_size - mem_cookie->offset)
-        size = mem_cookie->memory_size - mem_cookie->offset;
-    }
 
-  if (size > (mem_cookie->memory_size - mem_cookie->offset))
+  assert (mem_cookie->memory_size >= mem_cookie->offset);
+  nleft = mem_cookie->memory_size - mem_cookie->offset;
+  
+  /* If we are not allowed to grow limit the size to the left space.  */
+  if (!mem_cookie->flags.grow && size > nleft)
+    size = nleft;
+
+  /* Enlarge the memory buffer if needed.  */
+  if (size > nleft)
     {
       unsigned char *newbuf;
       size_t newsize;
-      
-      newsize = mem_cookie->memory_size + mem_cookie->block_size;
-      
-      newsize = mem_cookie->offset + size;
+
+      if (!mem_cookie->memory_size)
+        newsize = size;  /* Not yet allocated.  */
+      else
+        newsize = mem_cookie->memory_size + (nleft - size);
       if (newsize < mem_cookie->offset)
         {
           errno = EINVAL;
           return -1;
         }
-      newsize += mem_cookie->block_size - 1;
-      if (newsize < mem_cookie->offset)
+
+      /* Round up to the next block length.  BLOCK_SIZE should always
+         be set; we check anyway.  */
+      if (mem_cookie->block_size)
         {
-          errno = EINVAL;
-          return -1;
+          newsize += mem_cookie->block_size - 1;
+          if (newsize < mem_cookie->offset)
+            {
+              errno = EINVAL;
+              return -1;
+            }
+          newsize /= mem_cookie->block_size;
+          newsize *= mem_cookie->block_size;
         }
-      newsize /= mem_cookie->block_size;
-      newsize *= mem_cookie->block_size;
-      
+
+      /* Check for a total limit.  */
       if (mem_cookie->memory_limit && newsize > mem_cookie->memory_limit)
         {
           errno = ENOSPC;
@@ -491,8 +542,11 @@ es_func_mem_write (void *cookie, const void *buffer, size_t size)
       
       mem_cookie->memory = newbuf;
       mem_cookie->memory_size = newsize;
+
+      assert (mem_cookie->memory_size >= mem_cookie->offset);
+      nleft = mem_cookie->memory_size - mem_cookie->offset;
       
-      assert (!(size > (mem_cookie->memory_size - mem_cookie->offset)));
+      assert (size <= nleft);
     }
       
   memcpy (mem_cookie->memory + mem_cookie->offset, buffer, size);
@@ -540,7 +594,6 @@ es_func_mem_seek (void *cookie, off_t *offset, int whence)
 	{
 	  errno = ENOSPC;
 	  return -1;
-
         }
 
       newsize = pos_new + mem_cookie->block_size - 1;
@@ -551,6 +604,7 @@ es_func_mem_seek (void *cookie, off_t *offset, int whence)
         }
       newsize /= mem_cookie->block_size;
       newsize *= mem_cookie->block_size;
+
       if (mem_cookie->memory_limit && newsize > mem_cookie->memory_limit)
         {
           errno = ENOSPC;
@@ -2702,7 +2756,7 @@ es_getline (char *ES__RESTRICT *ES__RESTRICT lineptr, size_t *ES__RESTRICT n,
 
  out:
 
-  return err ? err : line_n;
+  return err ? err : (ssize_t)line_n;
 }
 
 
@@ -2797,7 +2851,9 @@ es_read_line (estream_t stream,
             {
               int save_errno = errno;
               mem_free (buffer); 
-              *length_of_buffer = *max_length = 0;
+              *length_of_buffer = 0;
+              if (max_length)
+                *max_length = 0;
               ESTREAM_UNLOCK (stream);
               errno = save_errno;
               return -1;
@@ -2873,6 +2929,44 @@ es_fprintf (estream_t ES__RESTRICT stream,
   return ret;
 }
 
+/* A variant of asprintf.  The function returns the allocated buffer
+   or NULL on error; ERRNO is set in the error case.  The caller
+   should use es_free to release the buffer.  This function actually
+   belongs into estream-printf but we put it here as a convenience
+   and because es_free is required anyway.  */
+char *
+es_asprintf (const char *ES__RESTRICT format, ...)
+{
+  int rc;
+  va_list ap;
+  char *buf;
+
+  va_start (ap, format);
+  rc = estream_vasprintf (&buf, format, ap);
+  va_end (ap);
+  if (rc < 0)
+    return NULL;
+  return buf;
+}
+
+
+/* A variant of vasprintf.  The function returns the allocated buffer
+   or NULL on error; ERRNO is set in the error case.  The caller
+   should use es_free to release the buffer.  This function actually
+   belongs into estream-printf but we put it here as a convenience
+   and because es_free is required anyway.  */
+char * 
+es_vasprintf (const char *ES__RESTRICT format, va_list ap)
+{
+  int rc;
+  char *buf;
+
+  rc = estream_vasprintf (&buf, format, ap);
+  if (rc < 0)
+    return NULL;
+  return buf;
+}
+
 
 static int
 tmpfd (void)
@@ -3203,7 +3297,7 @@ es_write_sanitized_utf8_buffer (estream_t stream,
         *bytes_written = strlen (buf);
       ret = es_fputs (buf, stream);
       xfree (buf);
-      return i;
+      return ret == EOF? ret : (int)i;
     }
   else
     return es_write_sanitized (stream, p, length, delimiters, bytes_written);
