@@ -1,6 +1,6 @@
 /* call-agent.c - Divert GPG operations to the agent.
  * Copyright (C) 2001, 2002, 2003, 2006, 2007, 
- *               2008 Free Software Foundation, Inc.
+ *               2008, 2009 Free Software Foundation, Inc.
  *
  * This file is part of GnuPG.
  *
@@ -39,12 +39,14 @@
 #include "asshelp.h"
 #include "sysutils.h"
 #include "call-agent.h"
+#include "status.h"
 
 #ifndef DBG_ASSUAN
 # define DBG_ASSUAN 1
 #endif
 
 static assuan_context_t agent_ctx = NULL;
+static int did_early_card_test;
 
 struct cipher_parm_s 
 {
@@ -75,36 +77,104 @@ struct genkey_parm_s
 };
 
 
+static int learn_status_cb (void *opaque, const char *line);
+
+
 
+/* If RC is not 0, write an appropriate status message. */
+static void
+status_sc_op_failure (int rc)
+{
+  switch (gpg_err_code (rc))
+    {
+    case 0:
+      break;
+    case GPG_ERR_CANCELED:
+      write_status_text (STATUS_SC_OP_FAILURE, "1");
+      break;
+    case GPG_ERR_BAD_PIN:
+      write_status_text (STATUS_SC_OP_FAILURE, "2");
+      break;
+    default:
+      write_status (STATUS_SC_OP_FAILURE);
+      break;
+    }
+}  
+
+
+
+
 /* Try to connect to the agent via socket or fork it off and work by
    pipes.  Handle the server's initial greeting */
 static int
-start_agent (void)
+start_agent (int for_card)
 {
   int rc;
 
+  /* Fixme: We need a context for each thread or serialize the access
+     to the agent. */
   if (agent_ctx)
-    return 0; /* Fixme: We need a context for each thread or serialize
-                 the access to the agent. */
-
-  rc = start_new_gpg_agent (&agent_ctx,
-                            GPG_ERR_SOURCE_DEFAULT,
-                            opt.homedir,
-                            opt.agent_program,
-                            opt.display, opt.ttyname, opt.ttytype,
-                            opt.lc_ctype, opt.lc_messages,
-                            opt.xauthority, opt.pinentry_user_data,
-                            opt.verbose, DBG_ASSUAN,
-                            NULL, NULL);
-  if (!rc)
+    rc = 0;
+  else
     {
-      /* Tell the agent that we support Pinentry notifications.  No
-         error checking so that it will work also with older
-         agents.  */
-      assuan_transact (agent_ctx, "OPTION allow-pinentry-notify",
-                       NULL, NULL, NULL, NULL, NULL, NULL);
+      rc = start_new_gpg_agent (&agent_ctx,
+                                GPG_ERR_SOURCE_DEFAULT,
+                                opt.homedir,
+                                opt.agent_program,
+                                opt.lc_ctype, opt.lc_messages,
+                                opt.session_env,
+                                opt.verbose, DBG_ASSUAN,
+                                NULL, NULL);
+      if (!rc)
+        {
+          /* Tell the agent that we support Pinentry notifications.
+             No error checking so that it will work also with older
+             agents.  */
+          assuan_transact (agent_ctx, "OPTION allow-pinentry-notify",
+                           NULL, NULL, NULL, NULL, NULL, NULL);
+        }
     }
 
+  if (!rc && for_card && !did_early_card_test)
+    {
+      /* Request the serial number of the card for an early test.  */
+      struct agent_card_info_s info;
+
+      memset (&info, 0, sizeof info);
+      rc = assuan_transact (agent_ctx, "SCD SERIALNO openpgp",
+                            NULL, NULL, NULL, NULL,
+                            learn_status_cb, &info);
+      if (rc)
+        {
+          switch (gpg_err_code (rc))
+            {
+            case GPG_ERR_NOT_SUPPORTED:
+            case GPG_ERR_NO_SCDAEMON:
+              write_status_text (STATUS_CARDCTRL, "6");
+              break;
+            default:
+              write_status_text (STATUS_CARDCTRL, "4");
+              log_info ("selecting openpgp failed: %s\n", gpg_strerror (rc));
+              break;
+            }
+        }
+
+      if (!rc && is_status_enabled () && info.serialno)
+        {
+          char *buf;
+          
+          buf = xasprintf ("3 %s", info.serialno);
+          write_status_text (STATUS_CARDCTRL, buf);
+          xfree (buf);
+        }
+
+      agent_release_card_info (&info);
+
+      if (!rc)
+        did_early_card_test = 1;
+    }
+
+  
   return rc;
 }
 
@@ -132,7 +202,6 @@ unhexify_fpr (const char *hexstr, unsigned char *fpr)
     ;
   if (*s || (n != 40))
     return 0; /* no fingerprint (invalid or wrong length). */
-  n /= 2;
   for (s=hexstr, n=0; *s; s += 2, n++)
     fpr[n] = xtoi_2 (s);
   return 1; /* okay */
@@ -170,6 +239,38 @@ dummy_data_cb (void *opaque, const void *buffer, size_t length)
   return 0;
 }
 
+/* A simple callback used to return the serialnumber of a card.  */
+static int
+get_serialno_cb (void *opaque, const char *line)
+{
+  char **serialno = opaque;
+  const char *keyword = line;
+  const char *s;
+  int keywordlen, n;
+
+  for (keywordlen=0; *line && !spacep (line); line++, keywordlen++)
+    ;
+  while (spacep (line))
+    line++;
+
+  if (keywordlen == 8 && !memcmp (keyword, "SERIALNO", keywordlen))
+    {
+      if (*serialno)
+        return gpg_error (GPG_ERR_CONFLICT); /* Unexpected status line. */
+      for (n=0,s=line; hexdigitp (s); s++, n++)
+        ;
+      if (!n || (n&1)|| !(spacep (s) || !*s) )
+        return gpg_error (GPG_ERR_ASS_PARAMETER);
+      *serialno = xtrymalloc (n+1);
+      if (!*serialno)
+        return out_of_core ();
+      memcpy (*serialno, line, n);
+      (*serialno)[n] = 0;
+    }
+  
+  return 0;
+}
+
 
 /* This is the default inquiry callback.  It mainly handles the
    Pinentry notifications.  */
@@ -297,6 +398,30 @@ learn_status_cb (void *opaque, const char *line)
           xfree (buf);
         }
     }
+  else if (keywordlen == 6 && !memcmp (keyword, "EXTCAP", keywordlen))
+    {
+      char *p, *p2, *buf;
+      int abool;
+
+      buf = p = unescape_status_string (line);
+      if (buf)
+        {
+          for (p = strtok (buf, " "); p; p = strtok (NULL, " "))
+            {
+              p2 = strchr (p, '=');
+              if (p2)
+                {
+                  *p2++ = 0;
+                  abool = (*p2 == '1');
+                  if (!strcmp (p, "ki"))
+                    parm->extcap.ki = abool;
+                  else if (!strcmp (p, "aac"))
+                    parm->extcap.aac = abool;
+                }
+            }
+          xfree (buf);
+        }
+    }
   else if (keywordlen == 7 && !memcmp (keyword, "KEY-FPR", keywordlen))
     {
       int no = atoi (line);
@@ -311,6 +436,20 @@ learn_status_cb (void *opaque, const char *line)
       else if (no == 3)
         parm->fpr3valid = unhexify_fpr (line, parm->fpr3);
     }
+  else if (keywordlen == 8 && !memcmp (keyword, "KEY-TIME", keywordlen))
+    {
+      int no = atoi (line);
+      while (* line && !spacep (line))
+        line++;
+      while (spacep (line))
+        line++;
+      if (no == 1)
+        parm->fpr1time = strtoul (line, NULL, 10);
+      else if (no == 2)
+        parm->fpr2time = strtoul (line, NULL, 10);
+      else if (no == 3)
+        parm->fpr3time = strtoul (line, NULL, 10);
+    }
   else if (keywordlen == 6 && !memcmp (keyword, "CA-FPR", keywordlen))
     {
       int no = atoi (line);
@@ -347,12 +486,25 @@ agent_learn (struct agent_card_info_s *info)
 {
   int rc;
 
-  rc = start_agent ();
+  rc = start_agent (1);
   if (rc)
     return rc;
 
+  /* Send the serialno command to initialize the connection.  We don't
+     care about the data returned.  If the card has already been
+     initialized, this is a very fast command.  The main reason we
+     need to do this here is to handle a card removed case so that an
+     "l" command in --card-edit can be used to show ta newly inserted
+     card.  We request the openpgp card because that is what we
+     expect. */
+  rc = assuan_transact (agent_ctx, "SCD SERIALNO openpgp",
+                        NULL, NULL, NULL, NULL, NULL, NULL);
+  if (rc)
+    return rc;
+
+
   memset (info, 0, sizeof *info);
-  rc = assuan_transact (agent_ctx, "LEARN --send",
+  rc = assuan_transact (agent_ctx, "SCD LEARN --force",
                         dummy_data_cb, NULL, default_inq_cb, NULL,
                         learn_status_cb, info);
   /* Also try to get the key attributes.  */
@@ -379,7 +531,7 @@ agent_scd_getattr (const char *name, struct agent_card_info_s *info)
     return gpg_error (GPG_ERR_TOO_LARGE);
   stpcpy (stpcpy (line, "SCD GETATTR "), name); 
 
-  rc = start_agent ();
+  rc = start_agent (1);
   if (rc)
     return rc;
 
@@ -429,12 +581,14 @@ agent_scd_setattr (const char *name,
     }
   *p = 0;
 
-  rc = start_agent ();
-  if (rc)
-    return rc;
+  rc = start_agent (1);
+  if (!rc)
+    {
+      rc = assuan_transact (agent_ctx, line, NULL, NULL, 
+                            default_inq_cb, NULL, NULL, NULL);
+    }
 
-  rc = assuan_transact (agent_ctx, line, NULL, NULL, 
-                        default_inq_cb, NULL, NULL, NULL);
+  status_sc_op_failure (rc);
   return rc;
 }
 
@@ -469,7 +623,7 @@ agent_scd_writecert (const char *certidstr,
   char line[ASSUAN_LINELENGTH];
   struct writecert_parm_s parms;
 
-  rc = start_agent ();
+  rc = start_agent (1);
   if (rc)
     return rc;
 
@@ -519,7 +673,7 @@ agent_scd_writekey (int keyno, const char *serialno,
 
   (void)serialno;
 
-  rc = start_agent ();
+  rc = start_agent (1);
   if (rc)
     return rc;
 
@@ -534,6 +688,7 @@ agent_scd_writekey (int keyno, const char *serialno,
   rc = assuan_transact (agent_ctx, line, NULL, NULL,
                         inq_writekey_parms, &parms, NULL, NULL);
 
+  status_sc_op_failure (rc);
   return rc;
 }
 
@@ -557,7 +712,7 @@ scd_genkey_cb (void *opaque, const char *line)
     {
       parm->fprvalid = unhexify_fpr (line, parm->fpr);
     }
-  if (keywordlen == 8 && !memcmp (keyword, "KEY-DATA", keywordlen))
+  else if (keywordlen == 8 && !memcmp (keyword, "KEY-DATA", keywordlen))
     {
       gcry_mpi_t a;
       const char *name = line;
@@ -584,6 +739,10 @@ scd_genkey_cb (void *opaque, const char *line)
     {
       parm->created_at = (u32)strtoul (line, NULL, 10);
     }
+  else if (keywordlen == 8 && !memcmp (keyword, "PROGRESS", keywordlen))
+    {
+      write_status_text (STATUS_PROGRESS, line);
+    }
 
   return 0;
 }
@@ -603,7 +762,7 @@ agent_scd_genkey (struct agent_card_genkey_s *info, int keyno, int force,
 
   (void)serialno;
 
-  rc = start_agent ();
+  rc = start_agent (1);
   if (rc)
     return rc;
 
@@ -624,9 +783,104 @@ agent_scd_genkey (struct agent_card_genkey_s *info, int keyno, int force,
                         NULL, NULL, default_inq_cb, NULL,
                         scd_genkey_cb, info);
   
+  status_sc_op_failure (rc);
   return rc;
 }
 
+
+
+
+/* Issue an SCD SERIALNO openpgp command and if SERIALNO is not NULL
+   ask the user to insert the requested card.  */
+gpg_error_t
+select_openpgp (const char *serialno)
+{
+  gpg_error_t err;
+
+  /* Send the serialno command to initialize the connection.  Without
+     a given S/N we don't care about the data returned.  If the card
+     has already been initialized, this is a very fast command.  We
+     request the openpgp card because that is what we expect. 
+
+     Note that an opt.limit_card_insert_tries of 1 means: No tries at
+     all whereas 0 means do not limit the number of tries.  Due to the
+     sue of a pinentry prompt with a cancel option we use it here in a
+     boolean sense.  */
+  if (!serialno || opt.limit_card_insert_tries == 1)
+    err = assuan_transact (agent_ctx, "SCD SERIALNO openpgp",
+                           NULL, NULL, NULL, NULL, NULL, NULL);
+  else
+    {
+      char *this_sn = NULL;
+      char *desc;
+      int ask;
+      char *want_sn;
+      char *p;
+      
+      want_sn = xtrystrdup (serialno);
+      if (!want_sn)
+        return gpg_error_from_syserror ();
+      p = strchr (want_sn, '/');
+      if (p)
+        *p = 0;
+
+      do 
+        {
+          ask = 0;
+          err = assuan_transact (agent_ctx, "SCD SERIALNO openpgp",
+                                 NULL, NULL, NULL, NULL, 
+                                 get_serialno_cb, &this_sn);
+          if (gpg_err_code (err) == GPG_ERR_CARD_NOT_PRESENT)
+            ask = 1; 
+          else if (gpg_err_code (err) == GPG_ERR_NOT_SUPPORTED)
+            ask = 2;
+          else if (err)
+            ;
+          else if (this_sn)
+            {
+              if (strcmp (want_sn, this_sn))
+                ask = 2;
+            }
+
+          xfree (this_sn);
+          this_sn = NULL;
+                  
+          if (ask)
+            {
+              char *formatted = NULL;
+              char *ocodeset = i18n_switchto_utf8 ();
+
+              if (!strncmp (want_sn, "D27600012401", 12) 
+                  && strlen (want_sn) == 32 )
+                formatted = xtryasprintf ("(%.4s) %.8s",
+                                          want_sn + 16, want_sn + 20);
+              
+              err = 0;
+              desc = xtryasprintf 
+                ("%s:\n\n"
+                 "  \"%s\"",
+                 ask == 1
+                 ? _("Please insert the card with serial number")
+                 : _("Please remove the current card and "
+                     "insert the one with serial number"),
+                 formatted? formatted : want_sn);
+              if (!desc)
+                err = gpg_error_from_syserror ();
+              xfree (formatted);
+              i18n_switchback (ocodeset);
+              if (!err)
+                err = gpg_agent_get_confirmation (desc);
+              xfree (desc);
+            }
+        }
+      while (ask && !err);
+      xfree (want_sn);
+    }
+
+  return err;
+}
+
+
 
 static int
 membuf_data_cb (void *opaque, const void *buffer, size_t length)
@@ -655,19 +909,17 @@ agent_scd_pksign (const char *serialno, int hashalgo,
   *r_buf = NULL;
   *r_buflen = 0;
 
-  rc = start_agent ();
+  rc = start_agent (1);
+  if (gpg_err_code (rc) == GPG_ERR_CARD_NOT_PRESENT
+      || gpg_err_code (rc) == GPG_ERR_NOT_SUPPORTED)
+    rc = 0; /* We check later.  */
   if (rc)
     return rc;
 
   if (indatalen*2 + 50 > DIM(line))
     return gpg_error (GPG_ERR_GENERAL);
 
-  /* Send the serialno command to initialize the connection. We don't
-     care about the data returned.  If the card has already been
-     initialized, this is a very fast command.  We request the openpgp
-     card because that is what we expect. */
-  rc = assuan_transact (agent_ctx, "SCD SERIALNO openpgp",
-                        NULL, NULL, NULL, NULL, NULL, NULL);
+  rc = select_openpgp (serialno);
   if (rc)
     return rc;
 
@@ -694,17 +946,18 @@ agent_scd_pksign (const char *serialno, int hashalgo,
   if (rc)
     {
       xfree (get_membuf (&data, &len));
-      return rc;
     }
-  *r_buf = get_membuf (&data, r_buflen);
+  else
+    *r_buf = get_membuf (&data, r_buflen);
 
-  return 0;
+  status_sc_op_failure (rc);
+  return rc;
 }
 
 
 /* Decrypt INDATA of length INDATALEN using the card identified by
    SERIALNO.  Return the plaintext in a nwly allocated buffer stored
-   at the address of R_BUF. 
+   at the address of R_BUF.
 
    Note, we currently support only RSA or more exactly algorithms
    taking one input data element. */
@@ -719,7 +972,10 @@ agent_scd_pkdecrypt (const char *serialno,
   size_t len;
 
   *r_buf = NULL;
-  rc = start_agent ();
+  rc = start_agent (1);
+  if (gpg_err_code (rc) == GPG_ERR_CARD_NOT_PRESENT
+      || gpg_err_code (rc) == GPG_ERR_NOT_SUPPORTED)
+    rc = 0; /* We check later.  */
   if (rc)
     return rc;
 
@@ -727,15 +983,10 @@ agent_scd_pkdecrypt (const char *serialno,
   if (indatalen*2 + 50 > DIM(line))
     return gpg_error (GPG_ERR_GENERAL);
 
-  /* Send the serialno command to initialize the connection. We don't
-     care about the data returned.  If the card has already been
-     initialized, this is a very fast command.  We request the openpgp
-     card because that is what we expect. */
-  rc = assuan_transact (agent_ctx, "SCD SERIALNO openpgp",
-                        NULL, NULL, NULL, NULL, NULL, NULL);
+  rc = select_openpgp (serialno);
   if (rc)
     return rc;
-
+  
   sprintf (line, "SCD SETDATA ");
   p = line + strlen (line);
   for (i=0; i < indatalen ; i++, p += 2 )
@@ -753,13 +1004,16 @@ agent_scd_pkdecrypt (const char *serialno,
   if (rc)
     {
       xfree (get_membuf (&data, &len));
-      return rc;
     }
-  *r_buf = get_membuf (&data, r_buflen);
-  if (!*r_buf)
-    return gpg_error (GPG_ERR_ENOMEM);
+  else
+    {
+      *r_buf = get_membuf (&data, r_buflen);
+      if (!*r_buf)
+        rc = gpg_error (GPG_ERR_ENOMEM);
+    }
 
-  return 0;
+  status_sc_op_failure (rc);
+  return rc;
 }
 
 
@@ -775,7 +1029,7 @@ agent_scd_readcert (const char *certidstr,
   size_t len;
 
   *r_buf = NULL;
-  rc = start_agent ();
+  rc = start_agent (1);
   if (rc)
     return rc;
 
@@ -823,7 +1077,7 @@ agent_scd_change_pin (int chvno, const char *serialno)
     reset = "--reset";
   chvno %= 100;
 
-  rc = start_agent ();
+  rc = start_agent (1);
   if (rc)
     return rc;
 
@@ -831,6 +1085,7 @@ agent_scd_change_pin (int chvno, const char *serialno)
   line[DIM(line)-1] = 0;
   rc = assuan_transact (agent_ctx, line, NULL, NULL,
                         default_inq_cb, NULL, NULL, NULL);
+  status_sc_op_failure (rc);
   return rc;
 }
 
@@ -844,15 +1099,17 @@ agent_scd_checkpin  (const char *serialno)
   int rc;
   char line[ASSUAN_LINELENGTH];
 
-  rc = start_agent ();
+  rc = start_agent (1);
   if (rc)
     return rc;
 
   snprintf (line, DIM(line)-1, "SCD CHECKPIN %s", serialno);
   line[DIM(line)-1] = 0;
-  return assuan_transact (agent_ctx, line,
-                          NULL, NULL,
-                          default_inq_cb, NULL, NULL, NULL);
+  rc = assuan_transact (agent_ctx, line,
+                        NULL, NULL,
+                        default_inq_cb, NULL, NULL, NULL);
+  status_sc_op_failure (rc);
+  return rc;
 }
 
 
@@ -889,7 +1146,7 @@ agent_get_passphrase (const char *cache_id,
 
   *r_passphrase = NULL;
 
-  rc = start_agent ();
+  rc = start_agent (0);
   if (rc)
     return rc;
 
@@ -960,7 +1217,7 @@ agent_clear_passphrase (const char *cache_id)
   if (!cache_id || !*cache_id)
     return 0;
 
-  rc = start_agent ();
+  rc = start_agent (0);
   if (rc)
     return rc;
 
@@ -969,3 +1226,31 @@ agent_clear_passphrase (const char *cache_id)
   return assuan_transact (agent_ctx, line, NULL, NULL,
                           default_inq_cb, NULL, NULL, NULL);
 }
+
+
+/* Ask the agent to pop up a confirmation dialog with the text DESC
+   and an okay and cancel button. */
+gpg_error_t
+gpg_agent_get_confirmation (const char *desc)
+{
+  int rc;
+  char *tmp;
+  char line[ASSUAN_LINELENGTH];
+
+  rc = start_agent (0);
+  if (rc)
+    return rc;
+
+  tmp = percent_plus_escape (desc);
+  if (!tmp)
+    return gpg_error_from_syserror ();
+  snprintf (line, DIM(line)-1, "GET_CONFIRMATION %s", tmp);
+  line[DIM(line)-1] = 0;
+  xfree (tmp);
+
+  rc = assuan_transact (agent_ctx, line, NULL, NULL,
+                        default_inq_cb, NULL, NULL, NULL);
+  return rc;
+}
+
+
