@@ -57,6 +57,10 @@ struct agent_card_info_s
     int algo;              /* Algorithm identifier.  */
     unsigned int nbits;    /* Supported keysize.  */
   } key_attr[3];      
+  struct {
+    unsigned int ki:1;     /* Key import available.  */
+    unsigned int aac:1;    /* Algorithm attributes are changeable.  */
+  } extcap;
 };
 
 struct agent_card_genkey_s {
@@ -130,6 +134,9 @@ gpg_error_t agent_get_passphrase (const char *cache_id,
 /* Send the CLEAR_PASSPHRASE command to the agent.  */
 gpg_error_t agent_clear_passphrase (const char *cache_id);
 
+/* Present the prompt DESC and ask the user to confirm.  */
+gpg_error_t gpg_agent_get_confirmation (const char *desc);
+
 
 #endif /*GNUPG_G10_CALL_AGENT_H*/
 
