@@ -16,7 +16,7 @@
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, see <http://www.gnu.org/licenses/>.
  *
- * $Id: apdu.h 5008 2009-05-13 17:12:00Z wk $
+ * $Id: apdu.h 5080 2009-07-16 15:54:59Z wk $
  */
 
 #ifndef APDU_H
@@ -94,6 +94,7 @@ int apdu_open_remote_reader (const char *portstr,
                              void *closefnc_value);
 int apdu_shutdown_reader (int slot);
 int apdu_close_reader (int slot);
+void apdu_prepare_exit (void);
 int apdu_enum_reader (int slot, int *used);
 unsigned char *apdu_get_atr (int slot, size_t *atrlen);
 
@@ -105,6 +106,8 @@ const char *apdu_strerror (int rc);
 int apdu_connect (int slot);
 int apdu_disconnect (int slot);
 
+int apdu_set_progress_cb (int slot, gcry_handler_progress_t cb, void *cb_arg);
+
 int apdu_activate (int slot);
 int apdu_reset (int slot);
 int apdu_get_status (int slot, int hang,
